from typing import Any, Optional

import click
from pytimeparse import parse


class TimeDeltaParamType(click.ParamType):  # type: ignore
    name = "timedelta"

    def convert(
        self, value: Any, param: Optional[click.Parameter], ctx: Optional[click.Context]
    ) -> Optional[Any]:
        try:
            return parse(value)
        except ValueError:
            self.fail(f"{value!r} is not a valid ISO datetime", param, ctx)


TIME_DELTA = TimeDeltaParamType()



from datetime import datetime, timedelta
from typing import Any, Dict


def _calc_diff(
    worked_time: timedelta,
    start_date: datetime,
    end_date: datetime,
    working_hours: timedelta,
    period: timedelta,
) -> timedelta:
    total_time = end_date - start_date
    total_goal_work_time = working_hours * (total_time / period)
    return worked_time - total_goal_work_time


def watson_overtime(
    watson_report: Dict[str, Any], working_hours: timedelta, period: timedelta
) -> timedelta:
    worked_time = timedelta(seconds=watson_report["time"])
    start_date = datetime.fromisoformat(watson_report["timespan"]["from"])
    end_date = datetime.fromisoformat(watson_report["timespan"]["to"])

    return _calc_diff(worked_time, start_date, end_date, working_hours, period)



try:
    import importlib.metadata

    version = importlib.metadata.version(__name__)
except ImportError:
    import importlib_metadata

    version = importlib_metadata.version(__name__)

__version__ = version



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['watson_overtime']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<9.0.0', 'humanize>=3.14.0,<5.0.0', 'pytimeparse>=1.1.8,<2.0.0']

extras_require = \
{':python_version <= "3.7"': ['importlib-metadata==4.2.0']}

entry_points = \
{'console_scripts': ['watson-overtime = watson_overtime.main:main']}

setup_kwargs = {
    'name': 'watson-overtime',
    'version': '0.3.0',
    'description': 'Check overtime in combination with the td-watson time tracker',
    'long_description': '[![Tests](https://github.com/flyingdutchman23/watson-overtime/workflows/Tests/badge.svg)](https://github.com/flyingdutchman23/watson-overtime/actions?workflow=Tests)\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/flyingdutchman23/watson-overtime/main.svg)](https://results.pre-commit.ci/latest/github/flyingdutchman23/watson-overtime/main)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n[![Python Versions](https://img.shields.io/pypi/pyversions/watson-overtime)](https://img.shields.io/pypi/pyversions/watson-overtime)\n\n\n# watson-overtime\n\n\n## Description\n\nThis is a simple tool to calculate if a set working time is fulfilled for a\ncertain period of time. Therefore, it uses the time tracking software\n[td-watson][https://pypi.org/project/td-watson/] as input.\n\n\n## Usage\n\nGenerate a `watson report` in JSON format. This command e.g. generates a watson\nreport for the current for the project `PROJECT`. Pipe the output to\n`watson-overtime`:\n```bash\nwatson report -w -p PROJECT --json | watson-overtime\n```\n',
    'author': 'Joris Clement',
    'author_email': 'flyingdutchman@posteo.eu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/flyingdutchman23/watson-overtime',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from datetime import timedelta
import json
import sys

import click
from humanize import naturaldelta

from . import __version__
from .click_timedelta import TIME_DELTA
from .watson_overtime import watson_overtime


def _build_work_diff_msg(diff: timedelta) -> str:
    msg = f"You are {naturaldelta(diff)} "
    if diff < timedelta(0):
        msg += "behind"
    else:
        msg += "ahead of"
    msg += " schedule."
    return msg


@click.command()
@click.version_option(version=__version__)
@click.option(
    "--watson-report",
    "-r",
    help="Read json style of watson report",
    type=click.File("r"),
    default=sys.stdin,
)
@click.option(
    "--working-hours",
    "-w",
    help="Amount of planned working time",
    type=TIME_DELTA,
    default="40 hours",
)
@click.option(
    "--period",
    "-p",
    help="Duration in which the amount of planned working time should be achieved",
    type=TIME_DELTA,
    default="1 week",
)
def main(
    watson_report: click.File, working_hours: timedelta, period: timedelta
) -> None:
    watson_report = json.load(watson_report)

    diff = watson_overtime(watson_report, working_hours, period)
    click.echo(_build_work_diff_msg(diff))


if __name__ == "__main__":
    main()
